'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global adblockIsPaused, getUnicodeUrl, frameData, myfiltersObj,
   ElementTypes, log */

function onBeforeHeadersReceivedHandler(details) {
  if (adblockIsPaused()) {
    return {};
  }
  const newDetails = details;
  newDetails.url = getUnicodeUrl(newDetails.url);
  const { tabId } = newDetails;

  const topFrame = frameData.get(tabId, 0);
  const subFrame = (newDetails.frameId !== 0 ? frameData.get(tabId, newDetails.frameId) : null);

  // If top frame is whitelisted, don't process anything
  if (topFrame && topFrame.whitelisted) {
    return {};
  }
  // If request comes from whitelisted subFrame and
  // top frame is not whitelisted, don't process the request
  if (subFrame && subFrame.whitelisted) {
    return {};
  }
  const frameDomain = frameData.get(tabId, newDetails.frameId).domain;
  // If |matchGeneric| is null, don't test request against blocking generic rules
  const matchGeneric = myfiltersObj.blocking.whitelist.matches(
    topFrame.url,
    ElementTypes.genericblock,
    topFrame.url,
  );

  const cspMatch = myfiltersObj.blocking.matches(
    newDetails.url,
    ElementTypes.csp,
    frameDomain,
    true,
    true,
    matchGeneric,
  );

  if (cspMatch && cspMatch.blocked && cspMatch.filter && cspMatch.filter.cspText) {
    details.responseHeaders.push({
      name: 'Content-Security-Policy',
      value: cspMatch.filter.cspText,
    });
    log('CSP Match', details.tabId, details.url, cspMatch);

    return { responseHeaders: details.responseHeaders };
  }
  return {};
}
